--=======================================================================================================================
print("Ottonian Decisions: loaded")
--=======================================================================================================================
-- UTILITIES
--=======================================================================================================================
-- MOD CHECKS
-------------------------------------------------------------------------------------------------------------------------
function JFD_IsEDActive()
	local iEDID = "1f941088-b185-4159-865c-472df81247b2"
	for _, mod in pairs(Modding.GetActivatedMods()) do
		if (mod.ID == iEDID) then
			return true
		end
	end
	return false
end
local bIsEDActive = JFD_IsEDActive()
-------------------------------------------------------------------------------------------------------------------------
-- UTILITIES
-------------------------------------------------------------------------------------------------------------------------
-- JFD_SendWorldEvent
function JFD_SendWorldEvent(playerID, description)
	local player = Players[playerID]
	local playerTeam = Teams[player:GetTeam()]
	local activePlayer = Players[Game.GetActivePlayer()]
	if (not player:IsHuman()) and playerTeam:IsHasMet(activePlayer:GetTeam()) then
		Players[Game.GetActivePlayer()]:AddNotification(NotificationTypes["NOTIFICATION_DIPLOMACY_DECLARATION"], description, "[COLOR_POSITIVE_TEXT]World Events[ENDCOLOR]", -1, -1)
	end
end 
--=======================================================================================================================
-- DECISIONS
--=======================================================================================================================
-- GLOBALS
-------------------------------------------------------------------------------------------------------------------------
local civilizationID  = GameInfoTypes.CIVILIZATION_THP_OTTONIAN
local mathCeil = math.ceil

-------------------------------------------------------------------------------------------------------------------------
-- Ottonian: Install Family in the Duchies
-------------------------------------------------------------------------------------------------------------------------
local bDuchy = GameInfoTypes.BUILDING_THP_OTTO_TRADE_ROUTE

local Decisions_THP_Ottonian_Duchies = {}
	Decisions_THP_Ottonian_Duchies.Name = "TXT_KEY_DECISIONS_THP_OTTONIAN_DUCHIES"
	Decisions_THP_Ottonian_Duchies.Desc = "TXT_KEY_DECISIONS_THP_OTTONIAN_DUCHIES_DESC"
	HookDecisionCivilizationIcon(Decisions_THP_Ottonian_Duchies, "CIVILIZATION_THP_OTTONIAN")
	Decisions_THP_Ottonian_Duchies.CanFunc = (
	function(player)
		if player:GetCivilizationType() ~= civilizationID then return false, false end
		if load(player, "Decisions_THP_Ottonian_Duchies") then
			Decisions_THP_Ottonian_Duchies.Desc = Locale.ConvertTextKey("TXT_KEY_DECISIONS_THP_OTTONIAN_DUCHIES_ENACTED_DESC")
			return false, false, true
		end
		local goldCost = mathCeil(400*iMod)
		Decisions_THP_Ottonian_Duchies.Desc = Locale.ConvertTextKey("TXT_KEY_DECISIONS_THP_OTTONIAN_DUCHIES_DESC", goldCost)
		if player:GetNumCities() < 5 then return true, false end
		if player:GetGold() < goldCost then return true, false end
		if player:GetNumResourceAvailable(iMagistrate, false) < 2 then return true, false end
		return true, true
	end
	)

	Decisions_THP_Ottonian_Duchies.DoFunc = (
	function(player)
		local playerID = player:GetID()
		local goldCost = mathCeil(400*iMod)
		player:ChangeNumResourceTotal(iMagistrate, -2)
		player:ChangeGold(-goldCost)
		player:GetCapitalCity():SetNumRealBuilding(bDuchy, 1)
		save(player, "Decisions_THP_Ottonian_Duchies", true)
		JFD_SendWorldEvent(playerID, Locale.ConvertTextKey("TXT_KEY_WORLD_EVENT_THP_OTTONIAN_DUCHIES", player:GetName(), player:GetCivilizationShortDescription(), player:GetCivilizationAdjective()))
	end
	)

	Decisions_THP_Ottonian_Duchies.Monitors = {}
	Decisions_THP_Ottonian_Duchies.Monitors[GameEvents.CityCaptureComplete] = (
	function(oldPlayerID, bCap, iX, iY)
		local pOldPlayer = Players[oldPlayerID]
		if load(pOldPlayer, "Decisions_THP_OttonianDuchies") then
			if pOldPlayer:IsAlive() and pOldPlayer:CountNumBuildings(bDuchy) < 1 then
				pOldPlayer:GetCapitalCity():SetNumRealBuilding(bDuchy, 1)
			end
		end
		local cCity = Map.GetPlot(iX, iY):GetPlotCity()
		if cCity:IsOriginalMajorCapital() then
			local pNewPlayer = Players[cCity:GetOwner()]
			if pNewPlayer:GetCapitalCity() == cCity then
				cCity:SetNumRealBuilding(bDuchy, 1)
			end
			if pNewPlayer:CountNumBuildings(bDuchy) > 1 then
				for cCheckedCity in pNewPlayer:Cities() do
					if cCheckedCity:IsHasBuilding(bDuchy) and cCheckedCity ~= cCity then
						cCheckedCity:SetNumRealBuilding(bDuchy, 0)
					end
				end
			end
		end
	end
	)

Decisions_AddCivilisationSpecific(civilizationID, "Decisions_THP_Ottonian_Duchies", Decisions_THP_Ottonian_Duchies)

-------------------------------------------------------------------------------------------------------------------------
-- Ottonian: Promote the Scriptoria
-------------------------------------------------------------------------------------------------------------------------
local bHasTableBeenFilled = false
local bWGuild = GameInfoTypes.BUILDING_WRITERS_GUILD
local sWriter = GameInfoTypes.SPECIALIST_WRITER
local tFaithBuildings = {}

function PopulateFaithBuildingsTable()
	bHasTableBeenFilled = true
	local iNum = 1
	for row in DB.Query("SELECT BuildingType FROM Building_YieldChanges WHERE YieldType = 'YIELD_FAITH' UNION SELECT BuildingType FROM Building_YieldModifiers WHERE YieldType = 'YIELD_FAITH'") do
		tFaithBuildings[iNum] = row.BuildingType
		iNum = iNum + 1
	end
end

function CheckToFillTable()
	for i = 0, GameDefines.MAX_MAJOR_CIVS - 1, 1 do
		local pPlayer = Players[i]
		if load(pPlayer, "THP_Ottonian_Script") then
			PopulateFaithBuildingsTable()
			break
		end
	end
end
Events.SequenceGameInitComplete.Add(CheckToFillTable)

local Decisions_THP_Ottonian_Script = {}
	Decisions_THP_Ottonian_Script.Name = "TXT_KEY_DECISIONS_THP_OTTONIAN_SCRIPT"
	Decisions_THP_Ottonian_Script.Desc = "TXT_KEY_DECISIONS_THP_OTTONIAN_SCRIPT_DESC"
	HookDecisionCivilizationIcon(Decisions_THP_Ottonian_Script, "CIVILIZATION_THP_OTTONIAN")
	Decisions_THP_Ottonian_Script.CanFunc = (
	function(player)
		if player:GetCivilizationType() ~= civilizationID then return false, false end
		if load(player, "Decisions_THP_Ottonian_Script") then
			Decisions_THP_Ottonian_Script.Desc = Locale.ConvertTextKey("TXT_KEY_DECISIONS_THP_OTTONIAN_SCRIPT_ENACTED_DESC")
			return false, false, true
		end
		local goldCost = mathCeil(600*iMod)
		Decisions_THP_Ottonian_Script.Desc = Locale.ConvertTextKey("TXT_KEY_DECISIONS_THP_OTTONIAN_SCRIPT_DESC", goldCost)
		if not player:HasCreatedReligion() then return true, false end
		if player:CountNumBuildings(bWGuild) < 1 then return true, false end
		if player:GetGold() < goldCost then return true, false end
		if player:GetNumResourceAvailable(iMagistrate, false) < 1 then return true, false end
		return true, true
	end
	)

	Decisions_THP_Ottonian_Script.DoFunc = (
	function(player)
		local playerID = player:GetID()
		local goldCost = mathCeil(600*iMod)
		player:ChangeNumResourceTotal(iMagistrate, -1)
		player:ChangeGold(-goldCost)
		if not bHasTableBeenFilled then PopulateFaithBuildingsTable() end
		save(player, "Decisions_THP_Ottonian_Script", true)
		JFD_SendWorldEvent(playerID, Locale.ConvertTextKey("TXT_KEY_WORLD_EVENT_THP_OTTONIAN_SCRIPT", player:GetName(), player:GetCivilizationShortDescription(), player:GetCivilizationAdjective()))
	end
	)

	Decisions_THP_Ottonian_Script.Monitors = {}
	Decisions_THP_Ottonian_Script.Monitors[GameEvents.PlayerDoTurn] = (
	function(playerID)
		local pPlayer = Players[playerID]
		if load(pPlayer, "Decisions_THP_Ottonian_Script") then
			for cCity in pPlayer:Cities() do
				for k, vBuilding in pairs(tFaithBuildings) do
					if cCity:IsHasBuilding(vBuilding) then
						cCity:ChangeSpecialistGreatPersonProgressTimes100(sWriter, 100)
					end
				end
			end
		end
	end
	)
	
Decisions_AddCivilisationSpecific(civilizationID, "Decisions_THP_Ottonian_Script", Decisions_THP_Ottonian_Script)

--
--
