-- ========= --
-- UTILITIES --
-- ========= --

local iPracticalNumCivs = (GameDefines.MAX_MAJOR_CIVS - 1)

function JFD_IsCivilisationActive(civilizationID)
	for iSlot = 0, iPracticalNumCivs, 1 do
		local slotStatus = PreGame.GetSlotStatus(iSlot)
		if (slotStatus == SlotStatus["SS_TAKEN"] or slotStatus == SlotStatus["SS_COMPUTER"]) then
			if PreGame.GetCivilization(iSlot) == civilizationID then
				return true
			end
		end
	end
	return false
end

function JFDGame_IsAAActive()
	for _, mod in pairs(Modding.GetActivatedMods()) do
		if mod.ID == "432bc547-eb05-4189-9e46-232dbde8f09f" then
			return true
		end
	end
	return false
end
local isAAActive = JFDGame_IsAAActive()

-- ======= --
-- DEFINES --
-- ======= --

include("AdditionalAchievementsUtility.lua")
include("FLuaVector.lua")

local pCiv = GameInfoTypes.CIVILIZATION_THP_OTTONIAN
local bIsActive = JFD_IsCivilisationActive(pCiv)

-- ================= --
-- GENERAL FUNCTIONS --
-- ================= --

local tCapitals = {}
local tCityReligions = {}

function TabulateCities()
	for i = 0, iPracticalNumCivs, 1 do
		local pPlayer = Players[i]
		for cCity in pPlayer:Cities() do
			if cCity:IsOriginalMajorCapital() then
				tCapitals[iTableNum] = cCity
			end
			tCityReligions[cCity] = cCity:GetReligiousMajority()
		end
	end
end

function Otto_CityFounded(playerID, iX, iY)
	local pPlayer = Players[playerID]
	local cCity = Map.GetPlot(iX, iY):GetPlotCity()
	if cCity:IsOriginalMajorCapital() then
		table.insert(tCapitals, cCity)
	end
	tCityReligions[cCity] = cCity:GetReligiousMajority()
end

if bIsActive then
	Events.SequenceGameInitComplete.Add(TabulateCities)
	GameEvents.PlayerCityFounded.Add(Otto_CityFounded)
end

-- ============================================ --
-- UA: GREAT ARTIST POINTS FROM CITY CONVERSION --
-- ============================================ --

local sArtist = GameInfoTypes.SPECIALIST_ARTIST
local iBaseNumPoints = 5

local tReligions = {}
function TabulateReligionsAtGameStart()
	for i = 0, iPracticalNumCivs, 1 do
		local pPlayer = Players[i]
		if pPlayer:HasCreatedReligion() then
			local iReligion = pPlayer:GetReligionCreatedByPlayer()
			tReligions[iReligion] = pPlayer
		end
	end
end

function AddNewReligionToTable(playerID, cityID, religionID)
	local pPlayer = Players[playerID]
	tReligions[religionID] = pPlayer
end

function OttoArtPointsOnConversion(playerID, religionID, iX, iY)
	local pReligionOwner = tReligions[religionID]
	if pReligionOwner and pReligionOwner:GetCivilizationType() == pCiv then
		local cCity = Map.GetPlot(iX, iY):GetPlotCity()
		if tCityReligions[cCity] == religionID then return end
		local iPop = cCity:GetPopulation()
		local cOttoCap = pReligionOwner:GetCapitalCity()
		if (cOttoCap) and (cOttoCap ~= cCity) then
			local iUpdateVal = (iPop * iBaseNumPoints) * 100
			cOttoCap:ChangeSpecialistGreatPersonProgressTimes100(sArtist, iUpdateVal)
			if pReligionOwner:IsHuman() then
				local iRealVal = iPop * iBaseNumPoints
				Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(iX, iY))), "Converted for " .. iRealVal .. " [ICON_GREAT_PEOPLE] points[ENDCOLOR]", 1)
			end
		end
		tCityReligions[cCity] = religionID
	end
end

if bIsActive then
	Events.SequenceGameInitComplete.Add(TabulateReligionsAtGameStart)
	GameEvents.ReligionFounded.Add(AddNewReligionToTable)
	GameEvents.CityConvertsReligion.Add(OttoArtPointsOnConversion)
end

-- =================================== --
-- UA: SCIENCE FROM RELIGIOUS CAPITALS --
-- =================================== --

function GetPlayerReligion(pPlayer)
	if pPlayer:HasCreatedReligion() then
		return pPlayer:GetReligionCreatedByPlayer()
	else
		local iCapReligion = pPlayer:GetCapitalCity():GetReligiousMajority()
		if iCapReligion then
			return iCapReligion
		else
			return -1
		end
	end
end

function ScienceForForeignCapitals(pPlayer)
	local iBaseScience = 1
	local iEra = pPlayer:GetCurrentEra()
	local iTotal = iBaseScience * (iEra + 1)
	return iTotal
end

function OttoCheckCapitalReligions(playerID)
	local pPlayer = Players[playerID]
	if pPlayer:GetCivilizationType() == pCiv then
		local rOttoReligion = GetPlayerReligion(pPlayer)
		if rOttoReligion > 0 then
			local iScienceUpdate = ScienceForForeignCapitals(pPlayer)
			local iTotalSci = 0
			for k, vCapital in pairs(tCapitals) do
				if pPlayer:GetCapitalCity() ~= vCapital then
					local rCheckedReligion = vCapital:GetReligiousMajority()
					if (rCheckedReligion > 0) and (rCheckedReligion == rOttoReligion) then
						iTotalSci = iTotalSci + iScienceUpdate
						local techCurrent = pPlayer:GetCurrentResearch()
						local tTeamTech = Teams[pPlayer:GetTeam()]:GetTeamTechs()
						tTeamTech:ChangeResearchProgress(techCurrent, iScienceUpdate, pPlayer)
					end
				end
			end
			if pPlayer:IsHuman() and iTotalSci > 0 then
				Events.GameplayAlertMessage("Gained " .. iTotalSci .. " [ICON_RESEARCH] Science from Capitals following your religion!")
			end
		end
	end
end

if bIsActive then
	GameEvents.PlayerDoTurn.Add(OttoCheckCapitalReligions)
end

-- ====================================== --
-- UU: STRONGER STACKED WITH RELIG. UNITS --
-- ====================================== --

local promoStacked = GameInfoTypes.PROMOTION_THP_LR_WITH_RELIGION
local ucSwordsman = GameInfoTypes.UNITCLASS_SWORDSMAN
local uLegioRegia = GameInfoTypes.UNIT_THP_LEGIO_REGIA

local tReligiousUnits = {}
for row in DB.Query("SELECT Type FROM Units WHERE ReligiousStrength > 0") do
	tReligiousUnits[GameInfoTypes[row.Type]] = 1
end

function GrantStackedPromo(uUnit)
	local bEarnedPromo = false
	local pPlot = uUnit:GetPlot()
	if pPlot:GetNumUnits() > 1 then
		for i = 0, pPlot:GetNumUnits() - 1, 1 do
			local uCheckedUnit = pPlot:GetLayerUnit(i)
			local uCheckedType = uCheckedUnit:GetUnitType()
			if tReligiousUnits[uCheckedType] == 1 then
				uUnit:SetHasPromotion(promoStacked, true)
				bEarnedPromo = true
				break
			end
		end
	end
	if not bEarnedPromo then
		uUnit:SetHasPromotion(promoStacked, false)
	end
end

function UpdateLegioRegia(playerID)
	local pPlayer = Players[playerID]
	if pPlayer:HasUnitOfClassType(ucSwordsman) then
		for uUnit in pPlayer:Units() do
			if uUnit:GetUnitType() == uLegioRegia then
				GrantStackedPromo(uUnit)
			end
		end
	end
end

function NewlyAcquiredLegioRegia(playerID, cityID, unitID)
	local pPlayer = Players[playerID]
	local uUnit = pPlayer:GetUnitByID(unitID)
	if uUnit:GetUnitType() == uLegioRegia then
		GrantStackedPromo(uUnit)
	elseif (tReligiousUnits[uUnit:GetUnitType()] == 1) and (pPlayer:HasUnitOfClassType(ucSwordsman)) then
		local pPlot = pPlayer:GetCityByID(cityID):Plot()
		if pPlot:GetNumUnits() > 1 then
			for i = 0, pPlot:GetNumUnits() - 1, 1 do
				local uOtherUnit = pPlot:GetLayerUnit(i)
				if uOtherUnit:GetUnitType() == uLegioRegia then
					GrantStackedPromo(uOtherUnit)
				end
			end
		end
	end
end

function Otto_RevokePromotionForUpgradedUU(playerID, oldUnitID, newUnitID)
	local pPlayer = Players[playerID]
	local uOldUnit = pPlayer:GetUnitByID(oldUnitID)
	local uNewUnit = pPlayer:GetUnitByID(newUnitID)
	if uOldUnit:GetUnitType() == uLegioRegia then
		if uNewUnit:IsHasPromotion(promoStacked) then
			uNewUnit:SetHasPromotion(promoStacked, false)
		end
	elseif uNewUnit:GetUnitType() == uLegioRegia then
		GrantStackedPromo(uNewUnit)
	end
end

if bIsActive then
	GameEvents.PlayerDoTurn.Add(UpdateLegioRegia)
	GameEvents.CityTrained.Add(NewlyAcquiredLegioRegia)
	GameEvents.UnitUpgraded.Add(Otto_RevokePromotionForUpgradedUU)
end

-- =================================== --
-- UU: ARTIST POINTS FROM CITY CAPTURE --
-- =================================== --

local iBaseArtPoints = 5

function OttoArtisticCapture(oldPlayerID, bCap, iX, iY, cityID, iOldPop, bConquest)
	if not bConquest then return end
	local pPlot = Map.GetPlot(iX, iY)
	local bIsUnitLR = false
	for i = 0, pPlot:GetNumUnits() - 1, 1 do
		local uUnit = pPlot:GetLayerUnit(i)
		if uUnit:GetUnitType() == uLegioRegia then
			bIsUnitLR = true
			break
		end
	end
	if bIsUnitLR then
		local cCapturedCity = pPlot:GetPlotCity()
		local pPlayer = Players[cCapturedCity:GetOwner()]
		local cCap = pPlayer:GetCapitalCity()
		if cCap then
			local iAddVal = (iBaseArtPoints * iOldPop) * 100
			cCap:ChangeSpecialistGreatPersonProgressTimes100(sArtist, iAddVal)
			if pPlayer:IsHuman() then
				local iRealVal = iAddVal / 100
				Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(cCapturedCity:GetX(), cCapturedCity:GetY()))), "+" .. iRealVal .. " [ICON_GREAT_PEOPLE] points", 1)
			end
		end
	end
end

if bIsActive then
	GameEvents.CityCaptureComplete.Add(OttoArtisticCapture)
end

-- ==================================== --
-- UB: SCIENCE FROM FOREIGN GREAT WORKS --
-- ==================================== --

local bKaiserpfalz = GameInfoTypes.BUILDING_THP_KAISERPFALZ
local bcCastle = GameInfoTypes.BUILDINGCLASS_CASTLE
local gwcArt = GameInfoTypes.GREAT_WORK_ART
local gwcArtifact = GameInfoTypes.GREAT_WORK_ARTIFACT

function CheckForKaiserpfalzScience(playerID)
	local pPlayer = Players[playerID]
	if pPlayer:CountNumBuildings(bKaiserpfalz) > 0 then
		for cCity in pPlayer:Cities() do
			local bIsSlotInfoKnown = false
			if cCity:IsHasBuilding(bKaiserpfalz) and cCity:IsHoldingGreatWork(bcCastle) then
				local bIsGain = false
				local iGreatWorkID = cCity:GetBuildingGreatWork(bcCastle, 0)
				local tPlayerArts = pPlayer:GetGreatWorks(gwcArt)
				local tPlayerArtifacts = pPlayer:GetGreatWorks(gwcArtifact)
				if iGreatWorkID > -1 then
					for k, v in pairs(tPlayerArts) do
						if v.Index == iGreatWorkID then
							bIsSlotInfoKnown = true
							if v.Creator ~= playerID then
								bIsGain = true
								local techCurrent = pPlayer:GetCurrentResearch()
								local tTeam = Teams[pPlayer:GetTeam()]
								tTeam:GetTeamTechs():ChangeResearchProgress(techCurrent, 2, pPlayer)
							end
							break
						end
					end
					if not bIsSlotInfoKnown then
						for k, v in pairs(tPlayerArtifacts) do
							if v.Index == iGreatWorkID then
								if v.Creator ~= playerID then
									bIsGain = true
									local techCurrent = pPlayer:GetCurrentResearch()
									local tTeam = Teams[pPlayer:GetTeam()]
									tTeam:GetTeamTechs():ChangeResearchProgress(techCurrent, 2, pPlayer)
								end
								break
							end
						end
					end
				end
				if bIsGain and pPlayer:IsHuman() then
									Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(cCity:GetX(), cCity:GetY()))), "Kaiserpfalz: +2 [ICON_RESEARCH] Science", 0)
				end
			end
		end
	end
end

if bIsActive then
	GameEvents.PlayerDoTurn.Add(CheckForKaiserpfalzScience)
end

-- ========== --
-- AA SUPPORT --
-- ========== --

function OttoAchievementCheck(playerID)
	if IsAAUnlocked('AA_THP_OTTONIAN_SPECIAL') then return end
	local pPlayer = Players[playerID]
	if pPlayer:IsHuman() and pPlayer:GetCivilizationType() == pCiv then
		for cCity in pPlayer:Cities() do
			if cCity:IsHolyCityAnyReligion() then
				local iNumGW = cCity:GetNumGreatWorks()
				if iNumGW > 4 then
					UnlockAA('AA_THP_OTTONIAN_SPECIAL');
				end
			end
		end
	end
end

if isAAActive then
	GameEvents.PlayerDoTurn.Add(OttoAchievementCheck)
end
